package com.brulijam.minecraftapi.telegrambot;

import com.brulijam.minecraftapi.model.Tutel;
import com.brulijam.minecraftapi.service.TelegramService;
import com.brulijam.minecraftapi.service.TutelService;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.telegram.telegrambots.bots.TelegramLongPollingBot;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import org.telegram.telegrambots.meta.api.objects.Update;
import org.telegram.telegrambots.meta.api.objects.User;
import org.telegram.telegrambots.meta.exceptions.TelegramApiException;

import java.util.List;

/*
Commands
help - Displays help
list_tutels - Lists all registered Tutels
get_tutel - Get Tutel by label
stop_tutel - Stop Tutel by label
*/

@Component
@RequiredArgsConstructor
@Getter
public class TelegramBot extends TelegramLongPollingBot {

    private final TutelService tutelService;
    private final TelegramService telegramService;

    @Value("${bot.name}")
    private String botUsername;
    @Value("${bot.token}")
    private String botToken;

    @Override
    public void onUpdateReceived(Update update) {

        if (update.hasMessage() && update.getMessage().hasText()) {
            User fromUser = update.getMessage().getFrom();

            String receivedMessage = update.getMessage().getText();


            handleMessage(fromUser, receivedMessage);


        }
    }

    private void handleMessage(User fromUser, String receivedMessage) {
        Long userId = fromUser.getId();

        if (receivedMessage.equals("/start")) {
            //check if user is already known
            if (telegramService.getTokens().containsKey(userId)) {
                sendMessage(userId, "You are already registered.");
            } else {
                telegramService.addUserToList(fromUser);
                String userToken = telegramService.getTokenByUserId(userId).replace("-", "");
                sendMessage(userId, "Your Telegram userId is " + userId + " and your access token is " + userToken);
            }

        } else if (receivedMessage.equals("/help")) {
            sendMessage(userId, "TODO List of commands");

        } else if (receivedMessage.equals("/list_tutels")) {
            List<Tutel> tutels = tutelService.getTutelsForTelegramUser(userId);
            sendMessage(userId,  tutelService.tutelsToJson(tutels).toString());

        } else if (receivedMessage.startsWith("/get_tutel_by_label ")) {
            String label = receivedMessage.substring(receivedMessage.indexOf(" "));
            Tutel tutel = tutelService.getTutelList().stream()
                    .filter(Tutel -> Tutel.getLabel().equals(label))
                    .findFirst()
                    .orElse(null);

        } else {
            sendMessage(userId, "I have no Idea what you want lol");
        }
    }

    @Override
    public String getBotUsername() {
        return botUsername;
    }

    @Override
    public String getBotToken() {
        return botToken;
    }

    public void sendMessage(Long chatId, String messageToSend) {
        SendMessage sendMessage = SendMessage.builder()
        .chatId(chatId.toString())
        .text(messageToSend)
        .build();

        try {
            execute(sendMessage);
        } catch (TelegramApiException e) {
            e.printStackTrace();
        }
    }
}
